@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -87,6 +87,7 @@ usage(void)
    printf("\n");
    printf("Options:\n");
    printf("  -f, --file FILE         Set the path to a user file\n");
+   printf("                          Defaults to %s", PGAGROAL_DEFAULT_USERS_FILE);
    printf("  -U, --user USER         Set the user name\n");
    printf("  -P, --password PASSWORD Set the password for the user\n");
    printf("  -g, --generate          Generate a password\n");
@@ -96,10 +97,11 @@ usage(void)
    printf("\n");
    printf("Commands:\n");
    printf("  master-key              Create or update the master key\n");
-   printf("  add-user                Add a user\n");
-   printf("  update-user             Update a user\n");
-   printf("  remove-user             Remove a user\n");
-   printf("  list-users              List all users\n");
+   printf("  user <subcommand>       Manage a specific user, where <subcommand> can be\n");
+   printf("                          - add  to add a new user\n");
+   printf("                          - del  to remove an existing user\n");
+   printf("                          - edit to change the password for an existing user\n");
+   printf("                          - ls   to list all available users\n");
    printf("\n");
    printf("pgagroal: %s\n", PGAGROAL_HOMEPAGE);
    printf("Report bugs: %s\n", PGAGROAL_ISSUES);
@@ -108,7 +110,6 @@ usage(void)
 int
 main(int argc, char** argv)
 {
-   int exit_code = 0;
    int c;
    char* username = NULL;
    char* password = NULL;
@@ -179,23 +180,47 @@ main(int argc, char** argv)
       {
          action = ACTION_MASTER_KEY;
       }
-      else if (!strcmp("add-user", argv[argc - 1]))
+      else if (parse_command_simple(argc, argv, optind, "user", "add")
+               || parse_deprecated_command(argc, argv, optind, "add-user", NULL, "user add", 1, 6))
       {
          action = ACTION_ADD_USER;
       }
-      else if (!strcmp("update-user", argv[argc - 1]))
+      else if (parse_command_simple(argc, argv, optind, "user", "edit")
+               || parse_deprecated_command(argc, argv, optind, "update-user", NULL, "user edit", 1, 6))
       {
          action = ACTION_UPDATE_USER;
       }
-      else if (!strcmp("remove-user", argv[argc - 1]))
+      else if (parse_command_simple(argc, argv, optind, "user", "del")
+               || parse_deprecated_command(argc, argv, optind, "remove-user", NULL, "user del", 1, 6))
       {
          action = ACTION_REMOVE_USER;
       }
-      else if (!strcmp("list-users", argv[argc - 1]))
+      else if (parse_command_simple(argc, argv, optind, "user", "ls")
+               || parse_deprecated_command(argc, argv, optind, "list-users", NULL, "user ls", 1, 6))
       {
          action = ACTION_LIST_USERS;
       }
 
+      // exit immediatly if the action is not understood!
+      if (action == ACTION_UNKNOWN)
+      {
+         warnx("unknown command or subcommand <%s>", argv[optind]);
+         usage();
+         goto error;
+      }
+
+      // if here, the action is understood, but we need
+      // the file to oeprate onto!
+      // Therefore, if the user did not specify any config file
+      // the default one is used. Note that in the case of ACTION_MASTER_KEY
+      // there is no need for the file_path to be set, so setting to a default
+      // value does nothing.
+      // Setting the file also means we don't have to check against the file_path value.
+      if (file_path == NULL)
+      {
+         file_path = PGAGROAL_DEFAULT_USERS_FILE;
+      }
+
       if (action == ACTION_MASTER_KEY)
       {
          if (master_key(password, generate_pwd, pwd_length))
@@ -205,69 +230,41 @@ main(int argc, char** argv)
       }
       else if (action == ACTION_ADD_USER)
       {
-         if (file_path != NULL)
+         if (add_user(file_path, username, password, generate_pwd, pwd_length))
          {
-            if (add_user(file_path, username, password, generate_pwd, pwd_length))
-            {
-               errx(1, "Error for add-user");
-            }
-         }
-         else
-         {
-            errx(1, "Missing file argument");
+            errx(1, "Error for <user add>");
          }
       }
       else if (action == ACTION_UPDATE_USER)
       {
-         if (file_path != NULL)
+         if (update_user(file_path, username, password, generate_pwd, pwd_length))
          {
-            if (update_user(file_path, username, password, generate_pwd, pwd_length))
-            {
-               errx(1, "Error for update-user");
-            }
-         }
-         else
-         {
-            errx(1, "Missing file argument");
+            errx(1, "Error for <user edit>");
          }
       }
       else if (action == ACTION_REMOVE_USER)
       {
-         if (file_path != NULL)
+
+         if (remove_user(file_path, username))
          {
-            if (remove_user(file_path, username))
-            {
-               errx(1, "Error for remove-user");
-            }
-         }
-         else
-         {
-            errx(1, "Missing file argument");
+            errx(1, "Error for <user del>");
          }
       }
       else if (action == ACTION_LIST_USERS)
       {
-         if (file_path != NULL)
+
+         if (list_users(file_path))
          {
-            if (list_users(file_path))
-            {
-               errx(1, "Error for list-users");
-            }
-         }
-         else
-         {
-            errx(1, "Missing file argument");
+            errx(1, "Error for <user ls>");
          }
+
       }
    }
 
-   if (action == ACTION_UNKNOWN)
-   {
-      usage();
-      exit_code = 1;
-   }
+   exit(0);
 
-   return exit_code;
+error:
+   exit(1);
 }
 
 static int
@@ -408,6 +405,7 @@ add_user(char* users_path, char* username, char* password, bool generate_pwd, in
 {
    FILE* users_file = NULL;
    char line[MISC_LENGTH];
+   char* entry = NULL;
    char* master_key = NULL;
    char* ptr = NULL;
    char* encrypted = NULL;
@@ -532,10 +530,14 @@ password:
    pgagroal_encrypt(password, master_key, &encrypted, &encrypted_length);
    pgagroal_base64_encode(encrypted, encrypted_length, &encoded);
 
-   snprintf(line, sizeof(line), "%s:%s\n", username, encoded);
+   entry = pgagroal_append(entry, username);
+   entry = pgagroal_append(entry, ":");
+   entry = pgagroal_append(entry, encoded);
+   entry = pgagroal_append(entry, "\n");
 
-   fputs(line, users_file);
+   fputs(entry, users_file);
 
+   free(entry);
    free(master_key);
    free(encrypted);
    free(encoded);
@@ -551,6 +553,7 @@ password:
 
 error:
 
+   free(entry);
    free(master_key);
    free(encrypted);
    free(encoded);
@@ -578,6 +581,7 @@ update_user(char* users_path, char* username, char* password, bool generate_pwd,
    char line_copy[MISC_LENGTH];
    char* master_key = NULL;
    char* ptr = NULL;
+   char* entry = NULL;
    char* encrypted = NULL;
    int encrypted_length = 0;
    char* encoded = NULL;
@@ -700,10 +704,14 @@ password:
          pgagroal_encrypt(password, master_key, &encrypted, &encrypted_length);
          pgagroal_base64_encode(encrypted, encrypted_length, &encoded);
 
-         memset(&line, 0, sizeof(line));
-         snprintf(line, sizeof(line), "%s:%s\n", username, encoded);
+         entry = NULL;
+         entry = pgagroal_append(entry, username);
+         entry = pgagroal_append(entry, ":");
+         entry = pgagroal_append(entry, encoded);
+         entry = pgagroal_append(entry, "\n");
 
-         fputs(line, users_file_tmp);
+         fputs(entry, users_file_tmp);
+         free(entry);
 
          found = true;
       }
@@ -905,8 +913,12 @@ generate_password(int pwd_length)
 
    s = pwd_length + 1;
 
-   pwd = malloc(s);
-   memset(pwd, 0, s);
+   pwd = calloc(1, s);
+   if (pwd == NULL)
+   {
+      pgagroal_log_fatal("Couldn't allocate memory while generating password");
+      return NULL;
+   }
 
    srand((unsigned)time(&t));
 
