@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -41,6 +41,16 @@ extern "C" {
  */
 #define PGAGROAL_MAIN_INI_SECTION "pgagroal"
 
+/*
+ * The following constants are used to clearly identify
+ * a section the user wants to get configuration
+ * or change. They are used in the config-get
+ * and config-set operations.
+ */
+#define PGAGROAL_CONF_SERVER_PREFIX "server"
+#define PGAGROAL_CONF_HBA_PREFIX    "hba"
+#define PGAGROAL_CONF_LIMIT_PREFIX  "limit"
+
 /**
  * Status that pgagroal_read_configuration() could provide.
  * Use only negative values for errors, since a positive return
@@ -241,6 +251,96 @@ pgagroal_init_pidfile_if_needed(void);
 bool
 pgagroal_can_prefill(void);
 
+/**
+ * Gets a configuration parameter and places into the string pointer.
+ * This is used, for example, to get a writable string to send over the
+ * management socket.
+ *
+ * The key can contain words separated by a dot '.' to indicate different search criterias.
+ * A "dotted" key is made by a 'context', a 'section' and a 'search term', so that
+ * it can be written as 'section.context.search'.
+ * If both the section and the context are omitted, the 'search' is performed among the
+ * pgagroal global settings (i.e., those under the [pgagroal] main section). The same
+ * happens if the the section is specified as 'pgagroal', therefore the following two
+ * terms do the same search:
+ * - `update_process_title`
+ * - `pgagroal.update_process_title`
+ *
+ * Other possible sections are:
+ * - server to search for a specific server, the match is performed on the server name;
+ * - hba to search for a specific HBA entry, the match is performed on the username;
+ * - limit to search for a specific database in the limit (database) configuration file.
+ *
+ * When one the above sections is specified, the search is done identifying the entry to snoop
+ * by means of 'context', and within such the 'search' is performed.
+ *
+ * In the case of the `server` section, the `context` has to be the name of a server configured,
+ * while the `search` has to be the keyword to look for. AS an example: `server.venkman.port` provides
+ * the value of the 'port' setting under the server section '[venkman]'.
+ *
+ * In the case of the 'hba` section, the `context` has to be a username as it appears in a line
+ * of the pgagroal_hba.conf file, while the `search` has to be the column keyword to snoop.
+ * For example, `hba.luca.method` will seek for the `method` used to authenticate the user `luca`.
+ * Please note that, since the same user could be listed more than once, only the first matching
+ * entry is reported.
+ *
+ * In the case of the 'limit` section, the `context` has to be a database name as it appears in a line
+ * of the pgagroal_database.conf file, while the `search` has to be the column keyword to snoop.
+ * For example, `limit.pgbench.max_size` will seek for the `max_size` connection limit for the
+ * database 'pgbench'.
+ * Please note that, since the same database could be listed more than once, only the first matching
+ * entry is reported.
+ *
+ * @param buffer where to write the configuration value. The buffer must
+ * be already allocated. In case of failure, the buffer is zero filled.
+ * @param config_key the name of the configuration parameter
+ * @param buffer_size the max length of the buffer where the result will be stored
+ * @return 0 on success, 1 when the key cannot be found
+ */
+int
+pgagroal_write_config_value(char* buffer, char* config_key, size_t buffer_size);
+
+/**
+ * Function to apply a single configuration parameter.
+ *
+ * This is the backbone function used when parsing the main configuration file
+ * and is used to set any of the allowed parameters.
+ *
+ * @param config the configuration to apply values onto
+ * @param srv the server to which the configuration parameter refers to, if needed
+ * @param section the section of the file, main or server
+ * @param key the parameter name of the configuration
+ * @param value the value of the configuration
+ * @return 0 on success
+ *
+ * Examples of usage:
+ *   pgagroal_apply_main_configuration( config, NULL, PGAGROAL_MAIN_INI_SECTION, "log_level", "info" );
+ */
+int
+pgagroal_apply_main_configuration(struct configuration* config,
+                                  struct server* srv,
+                                  char* section,
+                                  char* key,
+                                  char* value);
+
+/**
+ * Function to set a configuration value.
+ *
+ * This function accepts the same prefixes as the configuration get behavior, so
+ * a single parameter like 'max_connections' is managed as the main configuration file,
+ * a 'server' prefix will hit a specific server, a 'limit' prefix will set a limit, and so on.
+ *
+ * The idea behind the function is to "clone" the current configuration in use, and then
+ * apply the changes. In order to be coherent to what a "reload" operation would do,
+ * this function calls 'pgagroal_transfer_configuration' internally.
+ *
+ * @param config_key the string that contains the name of the parameter
+ * @param config_value the value to set
+ * @return 0 on success
+ */
+int
+pgagroal_apply_configuration(char* config_key, char* config_value);
+
 #ifdef __cplusplus
 }
 #endif
