@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -59,6 +59,21 @@ extern "C" {
 #define MANAGEMENT_SWITCH_TO          17
 #define MANAGEMENT_RELOAD             18
 #define MANAGEMENT_REMOVE_FD          19
+#define MANAGEMENT_CONFIG_GET         20
+#define MANAGEMENT_CONFIG_SET         21
+#define MANAGEMENT_CONFIG_LS          22
+
+/**
+ * Status for the 'ping' (i.e., is-alive) command
+ */
+#define PING_STATUS_RUNNING 1
+#define PING_STATUS_SHUTDOWN_GRACEFULLY 2
+
+/**
+ * Available command output formats
+ */
+#define COMMAND_OUTPUT_FORMAT_TEXT 'T'
+#define COMMAND_OUTPUT_FORMAT_JSON 'J'
 
 /**
  * Read the management header
@@ -176,10 +191,11 @@ pgagroal_management_status(SSL* ssl, int socket);
 /**
  * Management: Read status
  * @param socket The socket
+ * @param output_format a char describing the type of output (text or json)
  * @return 0 upon success, otherwise 1
  */
 int
-pgagroal_management_read_status(SSL* ssl, int socket);
+pgagroal_management_read_status(SSL* ssl, int socket, char output_format);
 
 /**
  * Management: Write status
@@ -202,10 +218,11 @@ pgagroal_management_details(SSL* ssl, int socket);
 /**
  * Management: Read details
  * @param socket The socket
+ * @param output_format the output format for this command (text, json)
  * @return 0 upon success, otherwise 1
  */
 int
-pgagroal_management_read_details(SSL* ssl, int socket);
+pgagroal_management_read_details(SSL* ssl, int socket, char output_format);
 
 /**
  * Management: Write details
@@ -230,7 +247,7 @@ pgagroal_management_isalive(SSL* ssl, int socket);
  * @return 0 upon success, otherwise 1
  */
 int
-pgagroal_management_read_isalive(SSL* ssl, int socket, int* status);
+pgagroal_management_read_isalive(SSL* ssl, int socket, int* status, char output_format);
 
 /**
  * Management: Write isalive
@@ -306,6 +323,143 @@ pgagroal_management_reload(SSL* ssl, int socket);
 int
 pgagroal_management_remove_fd(int32_t slot, int socket, pid_t pid);
 
+/**
+ * Management operation: get a configuration setting.
+ * This function sends over the socket the message to get a configuration
+ * value.
+ * In particular, the message block for the action config_get is sent,
+ * then the size of the configuration parameter to get (e.g., `max_connections`)
+ * and last the parameter name itself.
+ *
+ * @param ssl the SSL connection
+ * @param socket the socket file descriptor
+ * @param config_key the name of the configuration parameter to get back
+ * @return 0 on success, 1 on error
+ */
+int
+pgagroal_management_config_get(SSL* ssl, int socket, char* config_key);
+
+/**
+ * Management operation result: receives the key to read in the configuration.
+ *
+ * Internally, exploits the function to read the payload from the socket.
+ * @see pgagroal_management_read_payload
+ *
+ * @param ssl the socket file descriptor
+ * @param config_key the key to read (is used only to print in the output)
+ * @param verbose verbosity flag
+ * @param output_format the output format
+ * @param expected_value if set, a value that the configuration should match
+ * @return 0 on success
+ */
+int
+pgagroal_management_read_config_get(int socket, char* config_key, char* expected_value, bool verbose, char output_format);
+
+/**
+ * Management operation: write the result of a config_get action on the socket.
+ *
+ * Writes on the socket the result of the request for a specific
+ * configuration parameter.
+ *
+   ° @param socket the socket file descriptor
+ * @param config_key the name of the configuration parameter to get
+ * @return 0 on success
+ */
+int
+pgagroal_management_write_config_get(int socket, char* config_key);
+
+/**
+ * Management operation: set a configuration setting.
+ * This function sends over the socket the message to set a configuration
+ * value.
+ * In particular, the message block for the action config_set is sent,
+ * then the size of the configuration parameter to set (e.g., `max_connections`),
+ * then the parameter name. At this point another couple of "size" and "value" is
+ * sent with the size of the value to set and its value.
+ *
+ * @param ssl the SSL connection
+ * @param socket the socket file descriptor
+ * @param config_key the name of the configuration parameter to set
+ * @param config_value the value to set for the new parameter
+ * @return 0 on success, 1 on error
+ */
+int
+pgagroal_management_config_set(SSL* ssl, int socket, char* config_key, char* config_value);
+
+/**
+ * Function to execute the config-set and write over the socket
+ * the result.
+ *
+ * If the parameter is set, the function calls the
+ * pgagroal_management_write_config_get to send back over the
+ * socket the current value of the parameter. Therefore,
+ * this function answers always back the current value
+ * so that it is possible to reason about the new value and
+ * see if it has changed.
+ *
+ * @param socket the socket to use for communication
+ * @param config_key the key to set
+ * @param config_value the value to use
+ * @return 0 on success
+ */
+int
+pgagroal_management_write_config_set(int socket, char* config_key, char* config_value);
+
+/**
+ * Entry point for managing the `conf ls` command that
+ * will list all the configuration files used by the running
+ * daemon.
+ *
+ * @param ssl the SSL handler
+ * @param fd the socket file descriptor
+ * @returns 0 on success
+ */
+int
+pgagroal_management_conf_ls(SSL* ssl, int fd);
+
+/**
+ * Reads out of the socket the list of configuration
+ * files and prints them out to the standard output.
+ *
+ * The order of the read paths is:
+ * - configuration path
+ * - HBA path
+ * - limit path
+ * - frontend users path
+ * - admins path
+ * - Superusers path
+ * - users path
+ *
+ * @param socket the file descriptor of the open socket
+ * @param ssl the SSL handler
+ * @param output_format the format to output the command result
+ * @returns 0 on success
+ */
+int
+pgagroal_management_read_conf_ls(SSL* ssl, int socket, char output_format);
+
+/**
+ * The management function responsible for sending
+ * the configuration paths into the socket.
+ *
+ * The function sends every path following the path length,
+ * that must be limited to MAX_PATH size.
+ *
+ * The order of the sent paths is:
+ * - configuration path
+ * - HBA path
+ * - limit path
+ * - frontend users path
+ * - admins path
+ * - Superusers path
+ * - users path
+ *
+ * @params socket the file descriptor of the open socket
+ * @returns 0 on success
+ */
+int
+pgagroal_management_write_conf_ls(int socket);
+
 #ifdef __cplusplus
 }
 #endif
