@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -67,7 +67,7 @@ extern "C" {
 
 #define MAX_BUFFER_SIZE      65535
 #define DEFAULT_BUFFER_SIZE  65535
-#define SECURITY_BUFFER_SIZE   512
+#define SECURITY_BUFFER_SIZE  1024
 
 #define MAX_USERNAME_LENGTH   128
 #define MAX_DATABASE_LENGTH   256
@@ -92,15 +92,16 @@ extern "C" {
 
 #define NUMBER_OF_SECURITY_MESSAGES 5
 
-#define STATE_NOTINIT    -2
-#define STATE_INIT       -1
-#define STATE_FREE        0
-#define STATE_IN_USE      1
-#define STATE_GRACEFULLY  2
-#define STATE_FLUSH       3
-#define STATE_IDLE_CHECK  4
-#define STATE_VALIDATION  5
-#define STATE_REMOVE      6
+#define STATE_NOTINIT           -2
+#define STATE_INIT              -1
+#define STATE_FREE               0
+#define STATE_IN_USE             1
+#define STATE_GRACEFULLY         2
+#define STATE_FLUSH              3
+#define STATE_IDLE_CHECK         4
+#define STATE_MAX_CONNECTION_AGE 5
+#define STATE_VALIDATION         6
+#define STATE_REMOVE             7
 
 #define SECURITY_INVALID  -2
 #define SECURITY_REJECT   -1
@@ -141,6 +142,34 @@ extern "C" {
 #define UPDATE_PROCESS_TITLE_MINIMAL 2
 #define UPDATE_PROCESS_TITLE_VERBOSE 3
 
+/**
+ * Constants used to refer to an HBA entry field.
+ */
+#define PGAGROAL_HBA_ENTRY_TYPE "type"
+#define PGAGROAL_HBA_ENTRY_DATABASE "database"
+#define PGAGROAL_HBA_ENTRY_USERNAME "username"
+#define PGAGROAL_HBA_ENTRY_ADDRESS "address"
+#define PGAGROAL_HBA_ENTRY_METHOD "method"
+
+/**
+ * Constants used to refer to the limit structure fields
+ */
+#define PGAGROAL_LIMIT_ENTRY_DATABASE "database"
+#define PGAGROAL_LIMIT_ENTRY_USERNAME "username"
+#define PGAGROAL_LIMIT_ENTRY_MAX_SIZE "max_size"
+#define PGAGROAL_LIMIT_ENTRY_MIN_SIZE "min_size"
+#define PGAGROAL_LIMIT_ENTRY_INITIAL_SIZE "initial_size"
+#define PGAGROAL_LIMIT_ENTRY_LINENO "line_number"
+
+/**
+ * Constants used to manage the exit code
+ * of a command sent over the socket in the
+ * management stuff, e.g., `pgagroal-cli`.
+ */
+#define EXIT_STATUS_OK               0
+#define EXIT_STATUS_CONNECTION_ERROR 1
+#define EXIT_STATUS_DATA_ERROR       2
+
 #define likely(x)    __builtin_expect (!!(x), 1)
 #define unlikely(x)  __builtin_expect (!!(x), 0)
 
@@ -223,11 +252,15 @@ extern void* prometheus_cache_shmem;
  */
 struct server
 {
-   char name[MISC_LENGTH]; /**< The name of the server */
-   char host[MISC_LENGTH]; /**< The host name of the server */
-   int port;               /**< The port of the server */
-   bool tls;               /**< Use TLS if possible */
-   atomic_schar state;     /**< The state of the server */
+   char name[MISC_LENGTH];          /**< The name of the server */
+   char host[MISC_LENGTH];          /**< The host name of the server */
+   int port;                        /**< The port of the server */
+   bool tls;                        /**< Use TLS if possible */
+   char tls_cert_file[MISC_LENGTH]; /**< TLS certificate path */
+   char tls_key_file[MISC_LENGTH];  /**< TLS key path */
+   char tls_ca_file[MISC_LENGTH];   /**< TLS CA certificate path */
+   atomic_schar state;              /**< The state of the server */
+   int lineno;                      /**< The line number within the configuration file */
 } __attribute__ ((aligned (64)));
 
 /** @struct
@@ -251,6 +284,7 @@ struct connection
    int backend_secret; /**< The backend secret */
 
    signed char limit_rule; /**< The limit rule used */
+   time_t start_time;      /**< The start timestamp */
    time_t timestamp;       /**< The last used timestamp */
    pid_t pid;              /**< The associated process id */
    int fd;                 /**< The descriptor */
@@ -266,6 +300,7 @@ struct hba
    char username[MAX_USERNAME_LENGTH]; /**< The user name */
    char address[MAX_ADDRESS_LENGTH];   /**< The address / mask */
    char method[MAX_ADDRESS_LENGTH];    /**< The access method */
+   int lineno;                        /**< The line number within the configuration file */
 } __attribute__ ((aligned (64)));
 
 /** @struct
@@ -329,16 +364,17 @@ struct prometheus
    atomic_ulong session_time[HISTOGRAM_BUCKETS]; /**< The histogram buckets */
    atomic_ulong session_time_sum;                /**< Total session time */
 
-   atomic_ulong connection_error;       /**< The number of error calls */
-   atomic_ulong connection_kill;        /**< The number of kill calls */
-   atomic_ulong connection_remove;      /**< The number of remove calls */
-   atomic_ulong connection_timeout;     /**< The number of timeout calls */
-   atomic_ulong connection_return;      /**< The number of return calls */
-   atomic_ulong connection_invalid;     /**< The number of invalid calls */
-   atomic_ulong connection_get;         /**< The number of get calls */
-   atomic_ulong connection_idletimeout; /**< The number of idle timeout calls */
-   atomic_ulong connection_flush;       /**< The number of flush calls */
-   atomic_ulong connection_success;     /**< The number of success calls */
+   atomic_ulong connection_error;              /**< The number of error calls */
+   atomic_ulong connection_kill;               /**< The number of kill calls */
+   atomic_ulong connection_remove;             /**< The number of remove calls */
+   atomic_ulong connection_timeout;            /**< The number of timeout calls */
+   atomic_ulong connection_return;             /**< The number of return calls */
+   atomic_ulong connection_invalid;            /**< The number of invalid calls */
+   atomic_ulong connection_get;                /**< The number of get calls */
+   atomic_ulong connection_idletimeout;        /**< The number of idle timeout calls */
+   atomic_ulong connection_max_connection_age; /**< The number of max connection age calls */
+   atomic_ulong connection_flush;              /**< The number of flush calls */
+   atomic_ulong connection_success;            /**< The number of success calls */
 
    /**< The number of connection awaiting due to `blocking_timeout` */
    atomic_ulong connections_awaiting[NUMBER_OF_LIMITS];
@@ -421,6 +457,7 @@ struct configuration
 
    int blocking_timeout;         /**< The blocking timeout in seconds */
    int idle_timeout;             /**< The idle timeout in seconds */
+   int max_connection_age;       /**< The max connection age in seconds */
    int validation;               /**< Validation mode */
    int background_interval;      /**< Background validation timer in seconds */
    int max_retries;              /**< The maximum number of retries */
