@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -77,6 +77,12 @@ pgagroal_kill_connection(int slot, SSL* ssl);
 void
 pgagroal_idle_timeout(void);
 
+/**
+ * Perform max connection age check
+ */
+void
+pgagroal_max_connection_age(void);
+
 /**
  * Perform connection validation
  */
