@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -125,6 +125,12 @@ pgagroal_prometheus_connection_get(void);
 void
 pgagroal_prometheus_connection_idletimeout(void);
 
+/**
+ * Connection max connection age
+ */
+void
+pgagroal_prometheus_connection_max_connection_age(void);
+
 /**
  * Connection awaiting due to `blocking_timeout`.
  * Tracks the total awaiting connections and also the
