@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -402,7 +402,9 @@ retry:
          atomic_store(&config->log_lock, STATE_FREE);
       }
       else
-        SLEEP_AND_GOTO(1000000L,retry)
+      {
+         SLEEP_AND_GOTO(1000000L, retry)
+      }
    }
 }
 
@@ -428,7 +430,7 @@ retry:
 
       if (atomic_compare_exchange_strong(&config->log_lock, &isfree, STATE_IN_USE))
       {
-         char buf[256 * 1024];
+         char buf[(3 * size) + (2 * ((size / LINE_LENGTH) + 1)) + 1 + 1];
          int j = 0;
          int k = 0;
 
@@ -488,6 +490,8 @@ retry:
          atomic_store(&config->log_lock, STATE_FREE);
       }
       else
-        SLEEP_AND_GOTO(1000000L,retry)
+      {
+         SLEEP_AND_GOTO(1000000L, retry)
+      }
    }
 }
