@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -71,21 +71,18 @@ pgagroal_memory_size(size_t size)
 {
    pgagroal_memory_destroy();
 
-   message = (struct message*)malloc(sizeof(struct message));
+   message = (struct message*)calloc(1, sizeof(struct message));
    if (message == NULL)
    {
       goto error;
    }
 
-   data = malloc(size);
+   data = calloc(1, size);
    if (data == NULL)
    {
       goto error;
    }
 
-   memset(message, 0, sizeof(struct message));
-   memset(data, 0, size);
-
    message->kind = 0;
    message->length = 0;
    message->max_length = size;
