@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -44,6 +44,7 @@
 #include <unistd.h>
 #include <openssl/pem.h>
 #include <sys/types.h>
+#include <err.h>
 
 #ifndef EVBACKEND_LINUXAIO
 #define EVBACKEND_LINUXAIO 0x00000040U
@@ -54,7 +55,7 @@
 #endif
 
 extern char** environ;
-#ifdef HAVE_LINUX
+#if defined(HAVE_LINUX) || defined(HAVE_OSX)
 static bool env_changed = false;
 static int max_process_title_size = 0;
 #endif
@@ -108,8 +109,7 @@ pgagroal_extract_username_database(struct message* msg, char** username, char**
       end++;
       if (c == 0)
       {
-         array[counter] = (char*)malloc(end - start);
-         memset(array[counter], 0, end - start);
+         array[counter] = (char*)calloc(1, end - start);
          memcpy(array[counter], msg->data + start, end - start);
 
          start = end;
@@ -122,8 +122,7 @@ pgagroal_extract_username_database(struct message* msg, char** username, char**
       if (!strcmp(array[i], "user"))
       {
          size = strlen(array[i + 1]) + 1;
-         un = malloc(size);
-         memset(un, 0, size);
+         un = calloc(1, size);
          memcpy(un, array[i + 1], size);
 
          *username = un;
@@ -131,8 +130,7 @@ pgagroal_extract_username_database(struct message* msg, char** username, char**
       else if (!strcmp(array[i], "database"))
       {
          size = strlen(array[i + 1]) + 1;
-         db = malloc(size);
-         memset(db, 0, size);
+         db = calloc(1, size);
          memcpy(db, array[i + 1], size);
 
          *database = db;
@@ -140,8 +138,7 @@ pgagroal_extract_username_database(struct message* msg, char** username, char**
       else if (!strcmp(array[i], "application_name"))
       {
          size = strlen(array[i + 1]) + 1;
-         an = malloc(size);
-         memset(an, 0, size);
+         an = calloc(1, size);
          memcpy(an, array[i + 1], size);
 
          *appname = an;
@@ -230,8 +227,7 @@ pgagroal_extract_error_message(struct message* msg, char** error)
 
          if (type == 'M')
          {
-            result = (char*)malloc(strlen(s) + 1);
-            memset(result, 0, strlen(s) + 1);
+            result = (char*)calloc(1, strlen(s) + 1);
             memcpy(result, s, strlen(s));
 
             *error = result;
@@ -271,6 +267,8 @@ pgagroal_get_state_string(signed char state)
          return "Flush";
       case STATE_IDLE_CHECK:
          return "Idle check";
+      case STATE_MAX_CONNECTION_AGE:
+         return "Max connection age check";
       case STATE_VALIDATION:
          return "Validating";
       case STATE_REMOVE:
@@ -616,8 +614,7 @@ pgagroal_get_password(void)
 
    tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
 
-   result = malloc(strlen(p) + 1);
-   memset(result, 0, strlen(p) + 1);
+   result = calloc(1, strlen(p) + 1);
 
    memcpy(result, &p, strlen(p));
 
@@ -653,8 +650,7 @@ pgagroal_base64_encode(char* raw, int raw_length, char** encoded)
    BUF_MEM_grow(mem_bio_mem_ptr, (*mem_bio_mem_ptr).length + 1);
    (*mem_bio_mem_ptr).data[(*mem_bio_mem_ptr).length] = '\0';
 
-   r = malloc(strlen((*mem_bio_mem_ptr).data) + 1);
-   memset(r, 0, strlen((*mem_bio_mem_ptr).data) + 1);
+   r = calloc(1, strlen((*mem_bio_mem_ptr).data) + 1);
    memcpy(r, (*mem_bio_mem_ptr).data, strlen((*mem_bio_mem_ptr).data));
 
    BUF_MEM_free(mem_bio_mem_ptr);
@@ -685,8 +681,7 @@ pgagroal_base64_decode(char* encoded, size_t encoded_length, char** raw, int* ra
    }
 
    size = (encoded_length * 3) / 4 + 1;
-   decoded = malloc(size);
-   memset(decoded, 0, size);
+   decoded = calloc(1, size);
 
    b64_bio = BIO_new(BIO_f_base64());
    mem_bio = BIO_new(BIO_s_mem());
@@ -719,7 +714,7 @@ error:
 void
 pgagroal_set_proc_title(int argc, char** argv, char* s1, char* s2)
 {
-#ifdef HAVE_LINUX
+#if defined(HAVE_LINUX) || defined(HAVE_OSX)
    char title[MAX_PROCESS_TITLE_LENGTH];
    size_t size;
    char** env = environ;
@@ -751,14 +746,12 @@ pgagroal_set_proc_title(int argc, char** argv, char* s1, char* s2)
       for (int i = 0; env[i] != NULL; i++)
       {
          size = strlen(env[i]);
-         environ[i] = (char*)malloc(size + 1);
+         environ[i] = (char*)calloc(1, size + 1);
 
          if (environ[i] == NULL)
          {
             return;
          }
-
-         memset(environ[i], 0, size + 1);
          memcpy(environ[i], env[i], size);
       }
       environ[es] = NULL;
@@ -817,7 +810,7 @@ pgagroal_set_connection_proc_title(int argc, char** argv, struct connection* con
 {
    struct configuration* config;
    int primary;
-   char info[MAX_PROCESS_TITLE_LENGTH];
+   char* info = NULL;
 
    config = (struct configuration*)shmem;
 
@@ -828,13 +821,14 @@ pgagroal_set_connection_proc_title(int argc, char** argv, struct connection* con
       return;
    }
 
-   memset(&info, 0, sizeof(info));
-   snprintf(info, MAX_PROCESS_TITLE_LENGTH - 1, "%s@%s:%d",
-            connection->username,
-            config->servers[primary].host,
-            config->servers[primary].port);
+   info = pgagroal_append(info, connection->username);
+   info = pgagroal_append(info, "@");
+   info = pgagroal_append(info, config->servers[primary].host);
+   info = pgagroal_append(info, ":");
+   info = pgagroal_append_int(info, config->servers[primary].port);
 
    pgagroal_set_proc_title(argc, argv, info, connection->database);
+   free(info);
 }
 
 unsigned int
@@ -866,6 +860,74 @@ pgagroal_version_ge(unsigned int major, unsigned int minor, unsigned int patch)
    }
 }
 
+char*
+pgagroal_append(char* orig, char* s)
+{
+   size_t orig_length;
+   size_t s_length;
+   char* n = NULL;
+
+   if (s == NULL)
+   {
+      return orig;
+   }
+
+   if (orig != NULL)
+   {
+      orig_length = strlen(orig);
+   }
+   else
+   {
+      orig_length = 0;
+   }
+
+   s_length = strlen(s);
+
+   n = (char*)realloc(orig, orig_length + s_length + 1);
+
+   memcpy(n + orig_length, s, s_length);
+
+   n[orig_length + s_length] = '\0';
+
+   return n;
+}
+
+char*
+pgagroal_append_int(char* orig, int i)
+{
+   char number[12];
+
+   memset(&number[0], 0, sizeof(number));
+   snprintf(&number[0], 11, "%d", i);
+   orig = pgagroal_append(orig, number);
+
+   return orig;
+}
+
+char*
+pgagroal_append_ulong(char* orig, unsigned long l)
+{
+   char number[21];
+
+   memset(&number[0], 0, sizeof(number));
+   snprintf(&number[0], 20, "%lu", l);
+   orig = pgagroal_append(orig, number);
+
+   return orig;
+}
+
+char*
+pgagroal_append_ullong(char* orig, unsigned long long l)
+{
+   char number[21];
+
+   memset(&number[0], 0, sizeof(number));
+   snprintf(&number[0], 20, "%llu", l);
+   orig = pgagroal_append(orig, number);
+
+   return orig;
+}
+
 #ifdef DEBUG
 
 int
@@ -891,3 +953,158 @@ pgagroal_backtrace(void)
 }
 
 #endif
+
+bool
+parse_command(int argc,
+              char** argv,
+              int offset,
+              char* command,
+              char* subcommand,
+              char** key,
+              char* default_key,
+              char** value,
+              char* default_value)
+{
+
+   // sanity check: if no arguments, nothing to parse!
+   if (argc <= offset)
+   {
+      return false;
+   }
+
+   // first of all check if the command is the same
+   // as the first argument on the command line
+   if (strncmp(argv[offset], command, MISC_LENGTH))
+   {
+      return false;
+   }
+
+   if (subcommand)
+   {
+      // thre must be a subcommand check
+      offset++;
+
+      if (argc <= offset)
+      {
+         // not enough command args!
+         return false;
+      }
+
+      if (strncmp(argv[offset], subcommand, MISC_LENGTH))
+      {
+         return false;
+      }
+   }
+
+   if (key)
+   {
+      // need to evaluate the database or server or configuration key
+      offset++;
+      *key = argc > offset ? argv[offset] : default_key;
+      if (*key == NULL || strlen(*key) == 0)
+      {
+         goto error;
+      }
+
+      // do I need also a value?
+      if (value)
+      {
+         offset++;
+         *value = argc > offset ? argv[offset] : default_value;
+
+         if (*value == NULL || strlen(*value) == 0)
+         {
+            goto error;
+         }
+
+      }
+   }
+
+   return true;
+
+error:
+   return false;
+}
+
+bool
+parse_deprecated_command(int argc,
+                         char** argv,
+                         int offset,
+                         char* command,
+                         char** value,
+                         char* deprecated_by,
+                         unsigned int deprecated_since_major,
+                         unsigned int deprecated_since_minor)
+{
+   // sanity check: if no arguments, nothing to parse!
+   if (argc <= offset)
+   {
+      return false;
+   }
+
+   // first of all check if the command is the same
+   // as the first argument on the command line
+   if (strncmp(argv[offset], command, MISC_LENGTH))
+   {
+      return false;
+   }
+
+   if (value)
+   {
+      // need to evaluate the database or server
+      offset++;
+      *value = argc > offset ? argv[offset] : "*";
+   }
+
+   // warn the user if there is enough information
+   // about deprecation
+   if (deprecated_by
+       && pgagroal_version_ge(deprecated_since_major, deprecated_since_minor, 0))
+   {
+      warnx("command <%s> has been deprecated by <%s> since version %d.%d",
+            command, deprecated_by, deprecated_since_major, deprecated_since_minor);
+   }
+
+   return true;
+}
+
+bool
+parse_command_simple(int argc,
+                     char** argv,
+                     int offset,
+                     char* command,
+                     char* subcommand)
+{
+   return parse_command(argc, argv, offset, command, subcommand, NULL, NULL, NULL, NULL);
+}
+
+/**
+ * Given a server state, it returns a string that
+ * described the state in a human-readable form.
+ *
+ * If the state cannot be determined, the numeric
+ * form of the state is returned as a string.
+ *
+ * @param state the value of the sate for the server
+ * @returns the string representing the state
+ */
+char*
+pgagroal_server_state_as_string(signed char state)
+{
+   char* buf;
+
+   switch (state)
+   {
+      case SERVER_NOTINIT:  return "Not init";
+      case SERVER_NOTINIT_PRIMARY: return "Not init (primary)";
+      case SERVER_PRIMARY: return "Primary";
+      case SERVER_REPLICA: return "Replica";
+      case SERVER_FAILOVER: return "Failover";
+      case SERVER_FAILED: return "Failed";
+      default:
+         buf = malloc(5);
+         memset(buf, 0, 5);
+         snprintf(buf, 5, "%d", state);
+         return buf;
+   }
+}
