@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Red Hat
+ * Copyright (C) 2024 The pgagroal community
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
@@ -77,6 +77,7 @@ static void reload_cb(struct ev_loop* loop, ev_signal* w, int revents);
 static void graceful_cb(struct ev_loop* loop, ev_signal* w, int revents);
 static void coredump_cb(struct ev_loop* loop, ev_signal* w, int revents);
 static void idle_timeout_cb(struct ev_loop* loop, ev_periodic* w, int revents);
+static void max_connection_age_cb(struct ev_loop* loop, ev_periodic* w, int revents);
 static void validation_cb(struct ev_loop* loop, ev_periodic* w, int revents);
 static void disconnect_client_cb(struct ev_loop* loop, ev_periodic* w, int revents);
 static bool accept_fatal(int error);
@@ -310,6 +311,7 @@ main(int argc, char** argv)
    void* tmp_shmem = NULL;
    struct signal_info signal_watcher[6];
    struct ev_periodic idle_timeout;
+   struct ev_periodic max_connection_age;
    struct ev_periodic validation;
    struct ev_periodic disconnect_client;
    struct rlimit flimit;
@@ -841,9 +843,9 @@ read_superuser_path:
 #ifdef HAVE_LINUX
       sd_notifyf(0,
                  "STATUS=max_connections is larger than the file descriptor limit (%ld available)",
-                 flimit.rlim_cur - 30);
+                 (long)(flimit.rlim_cur - 30));
 #endif
-      errx(1, "max_connections is larger than the file descriptor limit (%ld available)", flimit.rlim_cur - 30);
+      errx(1, "max_connections is larger than the file descriptor limit (%ld available)", (long)(flimit.rlim_cur - 30));
    }
 
    if (daemon)
@@ -1020,6 +1022,13 @@ read_superuser_path:
       ev_periodic_start (main_loop, &idle_timeout);
    }
 
+   if (config->max_connection_age > 0)
+   {
+      ev_periodic_init (&max_connection_age, max_connection_age_cb, 0.,
+                        MAX(1. * config->max_connection_age / 2., 5.), 0);
+      ev_periodic_start (main_loop, &max_connection_age);
+   }
+
    if (config->validation == VALIDATION_BACKGROUND)
    {
       ev_periodic_init (&validation, validation_cb, 0.,
@@ -1293,8 +1302,12 @@ accept_main_cb(struct ev_loop* loop, struct ev_io* watcher, int revents)
    }
    else
    {
-      char* addr = malloc(strlen(address) + 1);
-      memset(addr, 0, strlen(address) + 1);
+      char* addr = calloc(1, strlen(address) + 1);
+      if (addr == NULL)
+      {
+         pgagroal_log_fatal("Cannot allocate memory for client address");
+         return;
+      }
       memcpy(addr, address, strlen(address));
 
       ev_loop_fork(loop);
@@ -1314,8 +1327,9 @@ accept_mgt_cb(struct ev_loop* loop, struct ev_io* watcher, int revents)
    int client_fd;
    signed char id;
    int32_t slot;
-   int payload_i;
+   int payload_i, secondary_payload_i;
    char* payload_s = NULL;
+   char* secondary_payload_s = NULL;
    struct configuration* config;
 
    if (EV_ERROR & revents)
@@ -1482,6 +1496,10 @@ accept_mgt_cb(struct ev_loop* loop, struct ev_io* watcher, int revents)
          pgagroal_log_debug("pgagroal: Management isalive");
          pgagroal_management_write_isalive(client_fd, config->gracefully);
          break;
+      case MANAGEMENT_CONFIG_LS:
+         pgagroal_log_debug("pgagroal: Management conf ls");
+         pgagroal_management_write_conf_ls(client_fd);
+         break;
       case MANAGEMENT_RESET:
          pgagroal_log_debug("pgagroal: Management reset");
          pgagroal_prometheus_reset();
@@ -1530,6 +1548,16 @@ accept_mgt_cb(struct ev_loop* loop, struct ev_io* watcher, int revents)
          pgagroal_log_debug("pgagroal: Management reload");
          reload_configuration();
          break;
+      case MANAGEMENT_CONFIG_GET:
+         pgagroal_log_debug("pgagroal: Management config-get for key <%s>", payload_s);
+         pgagroal_management_write_config_get(client_fd, payload_s);
+         break;
+      case MANAGEMENT_CONFIG_SET:
+         // this command has a secondary payload to extract, that is the configuration value
+         pgagroal_management_read_payload(client_fd, id, &secondary_payload_i, &secondary_payload_s);
+         pgagroal_log_debug("pgagroal: Management config-set for key <%s> setting value to <%s>", payload_s, secondary_payload_s);
+         pgagroal_management_write_config_set(client_fd, payload_s, secondary_payload_s);
+         break;
       default:
          pgagroal_log_debug("pgagroal: Unknown management id: %d", id);
          break;
@@ -1691,8 +1719,12 @@ accept_management_cb(struct ev_loop* loop, struct ev_io* watcher, int revents)
 
    if (!fork())
    {
-      char* addr = malloc(strlen(address) + 1);
-      memset(addr, 0, strlen(address) + 1);
+      char* addr = calloc(1, strlen(address) + 1);
+      if (addr == NULL)
+      {
+         pgagroal_log_fatal("Couldn't allocate address");
+         return;
+      }
       memcpy(addr, address, strlen(address));
 
       ev_loop_fork(loop);
@@ -1766,6 +1798,23 @@ idle_timeout_cb(struct ev_loop* loop, ev_periodic* w, int revents)
    }
 }
 
+static void
+max_connection_age_cb(struct ev_loop* loop, ev_periodic* w, int revents)
+{
+   if (EV_ERROR & revents)
+   {
+      pgagroal_log_trace("max_connection_age_cb: got invalid event: %s", strerror(errno));
+      return;
+   }
+
+   /* max_connection_age() is always in a fork() */
+   if (!fork())
+   {
+      shutdown_ports();
+      pgagroal_max_connection_age();
+   }
+}
+
 static void
 validation_cb(struct ev_loop* loop, ev_periodic* w, int revents)
 {
